DEFAULT_NAME = "OpenDota Hero Winrates"

DOTA_GRID = {
    "version": 3,
    "configs": [
        {
            "config_name": DEFAULT_NAME,
            "categories": [
                {
                    "category_name": "Strength",
                    "x_position": 0.000000,
                    "y_position": 0.000000,
                    "width": 1180.0,
                    "height": 180.0,
                    "hero_ids": [],
                },
                {
                    "category_name": "Agility",
                    "x_position": 0.000000,
                    "y_position": 200.000000,
                    "width": 1180.0,
                    "height": 180.0,
                    "hero_ids": [],
                },
                {
                    "category_name": "Intelligence",
                    "x_position": 0.000000,
                    "y_position": 400.000000,
                    "width": 1180.0,
                    "height": 180.0,
                    "hero_ids": [],
                }
            ],
        }
    ],
}

CATEGORY = {
    "category_name": "name",
    "x_position": 0.000000,
    "y_position": 0.000000,
    "width": 1180.0,
    "height": 180.0,
    "hero_ids": [],
}

CONFIG = {
    "config_name": DEFAULT_NAME,
    "categories": [
        {
            "category_name": "Strength",
            "x_position": 0.000000,
            "y_position": 0.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        },
        {
            "category_name": "Agility",
            "x_position": 0.000000,
            "y_position": 200.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        },
        {
            "category_name": "Intelligence",
            "x_position": 0.000000,
            "y_position": 400.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        }
    ]
}

CONFIG_TWO_CATEGORIES = {
    "config_name": DEFAULT_NAME,
    "categories": [
        {
            "category_name": "Melee",
            "x_position": 0.000000,
            "y_position": 0.000000,
            "width": 1180.0,
            "height": 280.0,
            "hero_ids": [],
        },
        {
            "category_name": "Ranged",
            "x_position": 0.000000,
            "y_position": 300.000000,
            "width": 1180.0,
            "height": 280.0,
            "hero_ids": [],
        },
    ]
}


CONFIG_THREE_CATEGORIES = {
    "config_name": DEFAULT_NAME,
    "categories": [
        {
            "category_name": "Strength",
            "x_position": 0.000000,
            "y_position": 0.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        },
        {
            "category_name": "Agility",
            "x_position": 0.000000,
            "y_position": 200.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        },
        {
            "category_name": "Intelligence",
            "x_position": 0.000000,
            "y_position": 400.000000,
            "width": 1180.0,
            "height": 180.0,
            "hero_ids": [],
        }
    ]
}


CONFIG_BASE = {
    "version": 3,
    "configs": [],
}
